@@ -29,13 +29,16 @@ class SchemaVersion(enum.Enum):
 
 
 class AccessType(enum.Enum):
-    OCI_REGISTRY = 'ociRegistry'
     GITHUB = 'github'
     HTTP = 'http'
-    LOCAL_OCI_BLOB = 'localOciBlob'
     LOCAL_FILESYSTEM_BLOB = 'localFilesystemBlob'
+    LOCAL_OCI_BLOB = 'localOciBlob'
+    OCI_REGISTRY = 'ociRegistry'
+    RELATIVE_OCI_REFERENCE = 'relativeOciReference'
+    S3 = 's3'
     NONE = 'None'  # the resource is only declared informally (e.g. generic)
 
+
 class SourceType(enum.Enum):
     GIT = 'git'
 
@@ -43,8 +46,7 @@ class SourceType(enum.Enum):
 class ResourceType(enum.Enum):
     OCI_IMAGE = 'ociImage'
     GENERIC = 'generic'
-    BLUEPRINT = 'blueprint'
-    HELM = 'helm'
+
 
 class ResourceRelation(enum.Enum):
     LOCAL = 'local'
@@ -61,6 +63,11 @@ class OciAccess(ResourceAccess):
     imageReference: str
 
 
+@dc(frozen=True)
+class RelativeOciAccess(ResourceAccess):
+    reference: str
+
+
 @dc(frozen=True)
 class GithubAccess(ResourceAccess):
     repoUrl: str
@@ -104,10 +111,18 @@ class GithubAccess(ResourceAccess):
 class HttpAccess(ResourceAccess):
     url: str
 
+
+@dc(frozen=True)
+class S3Access(ResourceAccess):
+    bucketName: str
+    objectKey: str
+
+
 @dc(frozen=True)
 class LocalOCIBlobAccess(ResourceAccess):
     digest: str
 
+
 @dc(frozen=True)
 class LocalFilesystemBlobAccess(ResourceAccess):
     filename: str
@@ -123,7 +138,7 @@ class Label:
 _no_default = object()
 
 
-class FindLabelMixin:
+class LabelMethodsMixin:
     def find_label(
         self,
         name: str,
@@ -140,6 +155,22 @@ class FindLabelMixin:
                 return None
             return default
 
+    def set_label(
+        self,
+        label: Label,
+        raise_if_present: bool = False,
+    ) -> typing.List[Label]:
+        if self.find_label(name=label.name) and raise_if_present:
+            raise ValueError(f'label {label.name} is already present')
+
+        patched_labels = [l for l in self.labels if l.name != label.name]
+        patched_labels.append(label)
+
+        return dataclasses.replace(
+            self,
+            labels=patched_labels,
+        )
+
 
 class Provider(enum.Enum):
     '''
@@ -276,7 +307,7 @@ class Artifact:
 
 
 @dc(frozen=True)
-class ComponentReference(Artifact, FindLabelMixin):
+class ComponentReference(Artifact, LabelMethodsMixin):
     name: str
     componentName: str
     version: str
@@ -285,18 +316,19 @@ class ComponentReference(Artifact, FindLabelMixin):
 
 
 @dc(frozen=True)
-class SourceReference(FindLabelMixin):
+class SourceReference(LabelMethodsMixin):
     identitySelector: typing.Dict[str, str]
     labels: typing.List[Label] = dataclasses.field(default_factory=tuple)
 
 
 @dc(frozen=True)
-class Resource(Artifact, FindLabelMixin):
+class Resource(Artifact, LabelMethodsMixin):
     name: str
     version: str
     type: typing.Union[ResourceType, str]
     access: typing.Union[
         OciAccess,
+        RelativeOciAccess,
         GithubAccess,
         HttpAccess,
         LocalOCIBlobAccess,
@@ -310,14 +342,25 @@ class Resource(Artifact, FindLabelMixin):
     srcRefs: typing.List[SourceReference] = dataclasses.field(default_factory=tuple)
 
 
-@dc
+@dc(frozen=True)
 class RepositoryContext:
+    pass  # actually, must have attr `type`
+
+
+class OciComponentNameMapping(enum.Enum):
+    URL_PATH = 'urlPath'
+    SHA256_DIGEST = 'sha256-digest'
+
+
+@dc(frozen=True)
+class OciRepositoryContext(RepositoryContext):
     baseUrl: str
+    componentNameMapping: OciComponentNameMapping = OciComponentNameMapping.URL_PATH
     type: AccessType = AccessType.OCI_REGISTRY
 
 
 @dc
-class ComponentSource(Artifact, FindLabelMixin):
+class ComponentSource(Artifact, LabelMethodsMixin):
     name: str
     access: GithubAccess
     version: typing.Optional[str] = None  # introduce this backwards-compatible for now
@@ -327,11 +370,11 @@ class ComponentSource(Artifact, FindLabelMixin):
 
 
 @dc
-class Component(FindLabelMixin):
+class Component(LabelMethodsMixin):
     name: str     # must be valid URL w/o schema
     version: str  # relaxed semver
 
-    repositoryContexts: typing.List[RepositoryContext]
+    repositoryContexts: typing.List[OciRepositoryContext]
     provider: Provider
 
     sources: typing.List[ComponentSource]
@@ -356,10 +399,10 @@ def _read_schema_file(schema_file_path: str):
 
 
 def enum_or_string(v, enum_type: enum.Enum):
-  try:
-    return enum_type(v)
-  except ValueError:
-    return str(v)
+    try:
+        return enum_type(v)
+    except ValueError:
+        return str(v)
 
 
 @dc
@@ -402,6 +445,7 @@ class ComponentDescriptor:
             data=component_descriptor_dict,
             config=dacite.Config(
                 cast=[
+                    OciComponentNameMapping,
                     AccessType,
                     Provider,
                     ResourceType,
@@ -412,6 +456,7 @@ class ComponentDescriptor:
                 type_hooks={
                     typing.Union[AccessType, str]: functools.partial(enum_or_string, enum_type=AccessType),
                     typing.Union[ResourceType, str]: functools.partial(enum_or_string, enum_type=ResourceType),
+                    typing.Union[OciComponentNameMapping, str]: functools.partial(enum_or_string, enum_type=OciComponentNameMapping),
                 },
             )
         )
